#!perl -w
use strict;
use Test::More tests => 44;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias LoadRootObject );
use DE_EPAGES::Object::API::Language qw( ExistsLanguageByCode GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Table::Language qw(
    DeleteLanguage
    InsertLanguage
);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle);

use constant ATTRIBUTE_PACKAGE => 'DE_EPAGES::Object::API::Attributes::LocalizedAttribute';

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), 'initialization' );

    my $Object = LoadRootObject->insertChild( 'C1', { Alias => 'C1O1' } );
    ok( $Object->id, 'InsertObject' );
    ok( $Object->class->existsAttribute( 'Description' ), 'existsAttribute' );

    eval {
        $Object->get('Description');
    };
    ok( ExistsError(), 'language id not set' );
    ok( GetError->code eq 'PARAMETER_MISSING', 'error code' );

    eval {
        $Object->set({'Description'=>'bla bla'});
    };
    ok( ExistsError(), 'language id not set' );
    ok( GetError->code eq 'PARAMETER_MISSING', 'error code' );

    my $EN = GetPKeyLanguageByCode( 'en' );
    ok( $EN, 'GetPKeyLanguageByCode en' );

    my $XXX = GetPKeyLanguageByCode( 'xxx' );
    ok( $XXX, 'GetPKeyLanguageByCode xxx' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $EN;

    $Object->set({
        Keywords => 'k',
        Description => 'd'
    });
    my $hAttributes = $Object->get( [qw( Description Keywords )] );
    ok( $hAttributes->{Keywords} eq 'k' && $hAttributes->{Description} eq 'd',
        'set/get multiple values' );

    my $KeywordsEN = 'MP3, Sex, Pics, Crack';
    $Object->set({'Keywords'=> $KeywordsEN});
    ok( $Object->get( 'Keywords' ) eq $KeywordsEN, 'set.../get... short text' );

    my $DescriptionEN = 'a asdas weq wrwe rewr werwe rwer werwerw er werw erweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr tttttttttttttttttttttttttttttttttttttt rerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww ende';
    $Object->set({'Description'=>$DescriptionEN});
    ok( $Object->get('Description') eq $DescriptionEN, 'set.../get... long text' );

    # update string with exactly 255 characters (256 bytes)
    $Object->set({'MyText'=> 'short'});
    use utf8;
    my $Description255 = '1.2gig celeron Processor<br>128mb Ram<br>20Gigbyte HardDisk Drive<br>52X CD-ROM (Max Speed)<br>Win 98 or win ME<br>56bps voice/fax Modem<br>Floppy Disk Drive<br>Mircrsoft Keyboard and Mouse Inc<br><br>Ä273.42 inc vat@ 17.5%<br><br>Add a monitor to this Gr';
    no utf8;
    $Object->set({'MyText'=>$Description255});
    ok( $Object->get('MyText') eq $Description255, 'set.../get... 255 text' );

    my $DescriptionXXX = 'a dasp aspd a+pda+sd a asdas weq wrwe rewr werwe rwer werwerw er werw erweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee rrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr tttttttttttttttttttttttttttttttttttttt rerrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww ende';
    $Object->set( { 'Description'=>$DescriptionXXX }, $XXX );
    ok( $Object->get( 'Description', $XXX ) eq $DescriptionXXX, 'set with language id' );
    ok( $Object->get( 'Description' ) eq $DescriptionEN, 'set with language id 2' );
    ok( $Object->get( 'Description', $XXX ) eq $DescriptionXXX, 'get single value' );
    ok( 2 == scalar keys %{$Object->get( ['Keywords','Description'] )}, 'get multiple values count' );
    ok( $Object->get( ['Keywords','Description'], $XXX )->{Description} eq $DescriptionXXX, 'get multiple values' );

    # local
    {
        local $DE_EPAGES::Object::API::Language::LANGUAGEID = $XXX;
        ok( $Object->get( 'Description' ) eq $DescriptionXXX, 'local LANGUAGEID 1' );
    }
    ok( $Object->get( 'Description' ) eq $DescriptionEN, 'local LANGUAGEID 2' );

    # allAttributes
    my $hAllAttributes = $Object->exportableAttributes;
    ok( $hAllAttributes->{Description}->{$EN} eq $DescriptionEN, 'allAttributes 1' );
    ok( $hAllAttributes->{Description}->{$XXX} eq $DescriptionXXX, 'allAttributes 2' );
    ok( $hAllAttributes->{Keywords}->{$EN} eq $KeywordsEN, 'allAttributes 3' );
    ok( !exists $hAllAttributes->{Keywords}->{$XXX}, 'allAttributes 4' );
    ok( !exists $hAllAttributes->{Comment}, 'allAttributes 5' );

    # empty string
    ok( !defined $Object->get('Comment'), 'empty string is not defined' );

    $Object->set({'Comment'=>'Test'});
    ok( $Object->get('Comment') eq 'Test', 'update empty string' );

    $Object->set({'Comment'=>''});
    ok( !defined $Object->get('Comment'), 'make non-empty string empty' );

    $Object->set({'Comment'=>'Test'});
    $Object->set({'Comment'=>undef});
    ok( !defined $Object->get('Comment'), 'set string to undef' );

    # spaces
    $Object->set({'Comment', ' '});
    is( $Object->get('Comment'), undef, 'single space' );

    my $dbh = GetCurrentDBHandle();
    if( $dbh->driverName eq 'mysql' ) {
        $Object->set({'Comment', ' -'});
        ok( $Object->get('Comment') eq ' -', 'leading space' );

        $Object->set({'Comment', '- '});
        ok( $Object->get('Comment') eq '- ', 'trailing space' );
    }
    else {
        $Object->set({'Comment', ' -'});
        ok( $Object->get('Comment') eq ' -', 'leading space' );

        $Object->set({'Comment', '- '});
        ok( $Object->get('Comment') eq '-', 'trailing space' );
    }

    $Object->set({'Comment', '  '});
    is( $Object->get('Comment'), undef, 'multiple spaces' );

    # UTF-8
    use utf8;
    my $ShortString = 'Радуга позволяет создавать';
    my $LongString = 'Радуга позволяет создавать плейлисты простым перемещением файлов из окна Проводника в главное окно Радуги. Вы можете изменять порядок проигрывания треков, перетаскивая их мышкой. Как только плейлист составлен, можно выходить в эфир. Просто нажмите кнопку \'Воспроизведение\' - и Вы в эфире!';
    no utf8;

    $Object->set({'Comment'=>$ShortString});
    ok( $Object->get('Comment') eq $ShortString, 'short UTF-8 string' );

    $Object->set({'Comment'=>$LongString});
    ok( $Object->get('Comment') eq $LongString, 'long UTF-8 string' );

    {
        use utf8;
        my $LanguageID = GetPKeyLanguageByCode('de');
        local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
        is( $Object->get( 'To' ), 'An', 'Get default value' );
        $Object->set({ 'To' => 'Empfänger' });
        is( $Object->get( 'To' ), 'Empfänger', 'Set new value' );
        is( MY::MailType_Subject->getAllAttributes( $Object )->{'To'}->{$LanguageID}, 'Empfänger',
            'New value is stored in the database.' );
        $Object->set({ 'To' => undef });
        is( $Object->get( 'To' ), 'An', 'Reset default value' );
        $Object->set({ 'To' => 'An' });
        is( $Object->get( 'To' ), 'An', 'Set to default value' );
        is( MY::MailType_Subject->getAllAttributes( $Object )->{'To'}->{$LanguageID}, undef,
            'Set to default value deletes the value from the database.' );
        no utf8;
    }
    is( $Object->get('Subject', $EN), 'Subject', 'English subject' );
    is( $Object->get('Subject', $XXX), undef, 'no default value provided' );

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }
    if( ExistsLanguageByCode( 'xxx' ) ) {
        DeleteLanguage( GetPKeyLanguageByCode( 'xxx' ) );
    };

    return 1;
}

sub Init {
    Cleanup();

    InsertLanguage( {
        English => 'TestLangauge_en',
        French => 'TestLangauge_fr',
        IsUsed => 1,
        Code2 => 'xx',
        Code3 => 'xxx',
        IsForBackoffice => 1
    } );

    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'Keywords', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'Description', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'Comment', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'MyText', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE });
    my $Subject = $Class1->insertAttribute({ Alias => 'Subject', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE});
    my $To = $Class1->insertAttribute({ Alias => 'To', Type => 'LocalizedString', Package => ATTRIBUTE_PACKAGE});
    $_->{'Attributes'}{'Package'} = 'MY::MailType_Subject' foreach( $Subject, $To );
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);

#========================================================================================
# §package      MY::MailType_Subject
# §base         DE_EPAGES::Object::API::Attributes::LocalizedAttribute
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements a sub class of DE_EPAGES::Object::API::Attributes::LocalizedAttribute
#               with an ovlerloaded defaultValue() method.
#========================================================================================
package MY::MailType_Subject;
use base ::ATTRIBUTE_PACKAGE();

use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID ); ## no critic (DE_EPAGES::Style::RequireCorrectImports)

use constant DEFAULTS => {
    'en' => { Subject => 'Subject', From => 'From' },
    'de' => { Subject => 'Betreff', To => 'An' },
};

sub defaultValue {
    shift;
    my ($Object, $Name, $LanguageID) = @_;

    my $Code2 = GetCodeByLanguageID( $LanguageID );
    return DEFAULTS->{$Code2}->{$Name};
}
