#!/bin/env perl

use strict;
use Cwd;
use File::Basename qw ( basename dirname );
use File::Find qw ( find );
use feature qw(say);

use DE_EPAGES::Core::API::File qw ( GetFileContent CopyDirectory RunOnDirectory WriteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw ( SystemCommand );
use DE_EPAGES::Database::API::Config qw( BUStoreTypes );
use DE_EPAGES::Dojo::API::Shrinksafe qw ( RunDojoShrinksafe );
use DE_EPAGES::JQuery::API::DictionaryConverter qw ( ConvertMainDictionary ConvertCartridgeDictionary ConvertSubDictionary );
use DE_EPAGES::Presentation::API::CSSPackageBuilder qw ( RunCSSPackaging );
use DE_EPAGES::Presentation::API::JSPackageBuilder qw ( RunJavaScriptPackaging );
use DE_EPAGES::XML::API::XML::CartridgeInstallerImportHandler qw(GetCartridges);

sub Main {
    local $| = 1;

    chdir dirname $0;
    chdir '..';

    my $Version = GetFileContent('version.txt');
    chomp $$Version;
    $$Version =~ s/^(([0-9]+\.){2}[0-9]+).*/$1/;

    my $SourceDir = 'FileChanges/srv/epages/eproot/Cartridges/DE_EPAGES';
    my $StoreTypeDir = "$ENV{EPAGES_WEBROOT}/StoreTypes/$$Version";

    my $FindStoreTypeChanges = $ENV{FIND_STORETYPE_CHANGES};
    $FindStoreTypeChanges = 0 if $^O eq 'MSWin32';

    my $StoreTypeChanges = cwd(). '/StoreTypeChanges';

    if ($FindStoreTypeChanges and -f "$StoreTypeChanges.tar.gz") {
        print "File found, won't recreate: $StoreTypeChanges.tar.gz\n";
        $FindStoreTypeChanges = 0;
    }
    my $Md5Sum = $FindStoreTypeChanges ? _GetMd5Sum($StoreTypeDir) : {};

    my $hStoreTypes = _GetStoreTypes();

    if (-d 'StoreTypeChanges') {
        foreach my $StoreType (sort keys %$hStoreTypes) {
            foreach my $VersionDir (glob "StoreTypeChanges/*/$hStoreTypes->{$StoreType}") {
                my $TargetDir = $VersionDir;
                $TargetDir =~ s,StoreTypeChanges/(.+?)/.*,StoreTypes/$1/$StoreType,;
                say qq(CopyDirectory('$VersionDir', "$ENV{EPAGES_WEBROOT}/$TargetDir"));
                CopyDirectory("$VersionDir", "$ENV{EPAGES_WEBROOT}/$TargetDir");
            }
        }
    }

    RunOnDirectory(
        $SourceDir,
        sub{
            foreach my $StoreType (sort keys %$hStoreTypes) {
                my $Files = _FindFiles($StoreType);

                say qq(CopyDirectory("*/Data/Public","$StoreTypeDir/$StoreType")) if $Files->{Public};
                CopyDirectory("$_/Data/Public","$StoreTypeDir/$StoreType") foreach @{$Files->{Public}};

                my $TargetDir = "$StoreTypeDir/$StoreType/javascript/epages/cartridges/de_epages/";
                say qq(CopyDirectory("*/Data/javascript","$TargetDir")) if $Files->{JS};
                CopyDirectory("$_/Data/javascript","$TargetDir/\L$_\Q") foreach @{$Files->{JS}};

                if ($Files->{MainDict}) {
                    say qq(ConvertMainDictionary({version => $$Version, storetype => $StoreType}));
                    ConvertMainDictionary({version => $$Version, storetype => $StoreType});
                }
                foreach (@{$Files->{CartDict}}) {
                    say qq(ConvertCartridgeDictionary({version => $$Version, storetype => $StoreType, cartridge => $_}));
                    ConvertCartridgeDictionary({version => $$Version, storetype => $StoreType, cartridge => $_});
                }
                foreach (@{$Files->{SubDict}}) {
                    say qq(ConvertSubDictionary({version => $$Version, storetype => $StoreType, cartridge => $_}));
                    ConvertSubDictionary({version => $$Version, storetype => $StoreType, cartridge => $_});
                }

                if (scalar @{$Files->{JS}} or $Files->{Dojo}) {
                    say qq(RunDojoShrinksafe({Version => $$Version, StoreType => $StoreType}));
                    RunDojoShrinksafe({Version => $$Version, StoreType => $StoreType});
                }
                if ($Files->{jQuery}) {
                    say qq(RunJavaScriptPackaging({Version => $$Version, StoreType => $StoreType}));
                    RunJavaScriptPackaging({Version => $$Version, StoreType => $StoreType});
                }
                if ($Files->{CSS}) {
                    say qq(RunCSSPackaging({Version => $$Version, StoreType => $StoreType}));
                    RunCSSPackaging({Version => $$Version, StoreType => $StoreType});
                }
            }
        }
    ) unless -d 'StoreTypeChanges';

    return unless $FindStoreTypeChanges;

    my $NewMd5Sum = _GetMd5Sum($StoreTypeDir);
    my $Content;
    foreach (sort keys %$NewMd5Sum) {
        next if defined $Md5Sum->{$_} and $NewMd5Sum->{$_} eq $Md5Sum->{$_};
        $Content .= "$_\n";
    }
    say qq(WriteFile("$StoreTypeChanges.txt", \$Content));
    WriteFile("$StoreTypeChanges.txt", \$Content);
    SystemCommand("cd '$ENV{EPAGES_WEBROOT}/StoreTypes' && cat '$StoreTypeChanges.txt' | tar cfT - - | gzip -c9 > '$StoreTypeChanges.tar.gz'",
                  { Print => 1 });
}

sub _GetStoreTypes {
    my %StoreTypes;

    foreach (@{BUStoreTypes()}) {
        $StoreTypes{$_} = 'Monitor' if /^Monitor/;
        $StoreTypes{$_} = 'Site' if /^Site/;
        $StoreTypes{$_} = 'Store' unless /^(Site|Monitor)/;
    }
    return \%StoreTypes;
}

sub _GetMd5Sum {
    my ($Dir) = @_;
    return {} if $^O eq 'MSWin32';
    say qq(_GetMd5Sum($Dir));

    my %Md5Sum;
    my $Parent = dirname $Dir;
    my $Base = basename $Dir;
    RunOnDirectory(
        $Parent,
        sub{
            foreach my $Line (split /\n/, SystemCommand("find $Base -type f -exec md5sum {} \\;",
                                                        { Print => 1, Capture => 1 })) {
                my ($Md5, $File) = split /\s+/, $Line;
                $Md5Sum{$File} = $Md5;
            }
        }
    );
    return \%Md5Sum;
}

sub _FindFiles {
    my ($StoreType) = @_;

    my $Files;
    my @Cartridges = grep { s,DE_EPAGES::,,; } @{GetCartridges()->{$StoreType}};

    find(
        sub{
            return unless -f;
            my $FN = $File::Find::name;
            if ($FN =~ qr(^\./([^/]+)/Data/Public/)) {
                return unless $1 ~~ @Cartridges;
                push @{$Files->{Public}}, $1 unless $1 ~~ @{$Files->{Public}};
            }
            if ($FN =~ qr(^\./([^/]+)/Data/javascript/)) {
                return unless $1 ~~ @Cartridges;
                push @{$Files->{JS}}, $1 unless $1 ~~ @{$Files->{JS}};
            }
            if ($FN =~ qr(^\./Dojo/)) {
                $Files->{Dojo} = 1;
            }
            if ($FN =~ qr(^\./(JQuery|Dictionary)/Templates/Dictionary\.[^/]+\.xml$)) {
                $Files->{MainDict} = 1;
            }
            elsif ($FN =~ qr(^\./([^/]+)/Templates/Dictionary\.[^/]+\.xml$)) {
                return unless $1 ~~ @Cartridges;
                push @{$Files->{CartDict}}, $1 unless $1 ~~ @{$Files->{CartDict}};
            }
            if ($FN =~ qr(^\./([^/]+)/Data/Public/GUI/de_epages/.+\.xml$)) {
                return unless $1 ~~ @Cartridges;
                push @{$Files->{SubDict}}, $1 unless $1 ~~ @{$Files->{SubDict}};
            }
            if ($FN =~ qr(^\./(JQuery|([^/]+)/Data/Public/GUI/.+\.(js|tmpl\.html)$))) {
                $Files->{jQuery} = 1;
            }
            if ($FN =~ qr(^\./([^/]+)/Data/Public/GUI/.+\.css$)) {
                return unless $1 ~~ @Cartridges;
                $Files->{CSS} = 1;
            }
        }, '.'
    );
    return $Files;
}

RunScript(
    'Sub' => \&Main
);
